﻿using System;
using System.ComponentModel;
using System.Data.SqlClient;
using Microsoft.VisualStudio.TestTools.WebTesting;

namespace TestUtil
{
   [DisplayName("SQL Validation Rule")]
   [Description("Executes the query and compares the first column of the first row to the Expected value.")]
   public class SQLValidationRule : ValidationRule
   {
      public SQLValidationRule()
      {
         IgnoreCase = true;
      }

      [Description("The query to execute and extract the first column from. In Select * from x where column={0}. You can leave the where clause if it is not needed.")]
      public string Query { get; set; }

      [DisplayName("Connection String")]
      [Description("The full connection string to the database")]
      public string ConnectionString { get; set; }

      [DisplayName("Where Clause Context Parameter Name")]
      [Description("The name of an optional context parameter to use if there is a where clause in the query.")]
      public string WhereClauseContextParameterName { get; set; }

      [DefaultValue(true)]
      [DisplayName("Ignore Case")]
      [Description("When set to true the case of the word is not used")]
      public bool IgnoreCase { get; set; }

      [DisplayName("Expected Value")]
      [Description("The value to compare the first column too.")]
      public string ExpectedValue { get; set; }

      public override void Validate(object sender, ValidationEventArgs e)
      {
         string where = null;
         if(!string.IsNullOrEmpty(WhereClauseContextParameterName))
            where = e.WebTest.Context[WhereClauseContextParameterName].ToString();

         string result = ExecuteQuery(ConnectionString, Query, where);

         e.IsValid = string.Compare(result, ExpectedValue, IgnoreCase) == 0;
      }

      private string ExecuteQuery(string connectionString, string query, string where)
      {
         SqlConnection conn = new SqlConnection(connectionString);

         string cmdText = string.Format(query, where);
         SqlCommand cmd = new SqlCommand(cmdText, conn);

         try
         {
            conn.Open();
            SqlDataReader dr = cmd.ExecuteReader();

            if(dr.Read())
               return dr.GetValue(0).ToString();
         }
         catch(Exception e)
         {
            System.Diagnostics.Debug.WriteLine(e.Message);
         }
         finally
         {
            conn.Close();
         }

         return null;
      }
   }
}
